@@ -1,11 +1,9 @@
 /*
-  $Id: Configuration.h 17617 2014-09-08 08:46:52Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/libticcutils/trunk/include/ticcutils/Configuration.h $
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  Copyright (c) 2006 - 2013
-  Tilburg University
-
-  This file is part of ticcutils.
+  This file is part of ticcutils
 
   ticcutils is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -18,12 +16,12 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+  along with this program; if not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/ticcutils/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
 */
 
 #ifndef TICC_CONFIGURATION_H
@@ -49,11 +47,30 @@ namespace TiCC {
     std::string setatt( const std::string&,
 			const std::string&,
 			const std::string& = "" );
+    std::string clearatt( const std::string&,
+			  const std::string& = "" );
     void dump( std::ostream& ) const;
-    std::string configDir() const { return cdir; };
+    void create_configfile( const std::string& ) const;
+    std::string configDir() const { return lookUp( "configDir" ); };
   private:
     sssMap myMap;
-    std::string cdir;
+    std::string cdir; // not used. left here for binary compatibility
   };
+
+  inline std::ostream& operator<<( std::ostream& os, const Configuration& c ){
+    c.dump(os);
+    return os;
+  }
+
+  inline std::ostream& operator<<( std::ostream& os, const Configuration* c ){
+    if ( c ){
+      c->dump(os);
+    }
+    else {
+      os << "empty config";
+    }
+    return os;
+  }
+
 }
 #endif
