@@ -1,10 +1,9 @@
 /*
-  $Id: Configuration.cxx 17617 2014-09-08 08:46:52Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/libticcutils/trunk/src/Configuration.cxx $
-  Copyright (c) 2006 - 2014
-  Tilburg University
+  Copyright (c) 2006 - 2016
+  CLST  - Radboud University
+  ILK   - Tilburg University
 
-  This file is part of ticcutils.
+  This file is part of ticcutils
 
   ticcutils is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,12 +16,13 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+  along with this program; if not, see <http://www.gnu.org/licenses/>.
 
   For questions and suggestions, see:
-      http://ilk.uvt.nl/software.html
+      https://github.com/LanguageMachines/ticcutils/issues
   or send mail to:
-      timbl@uvt.nl
+      lamasoftware (at ) science.ru.nl
+
 */
 
 #include <string>
@@ -88,13 +88,20 @@ namespace TiCC {
   }
 
   bool Configuration::fill( const string& fileName ){
-    cdir = dirname( fileName ) + "/";
-    //  cerr << "dirname= " << cdir << endl;
-    ifstream is( fileName.c_str() );
+    ifstream is( fileName );
     if ( !is ){
       cerr << "unable to read configuration from " << fileName << endl;
       return false;
     }
+    string cdir = dirname( fileName );
+    if ( cdir == "." ){
+      cdir = "";
+    }
+    else {
+      cdir += "/";
+    }
+    //  cerr << "dirname= " << cdir << endl;
+    myMap["global"]["configDir"] = cdir; // can be overidden below
     string inLine;
     string section = "global";
     while ( getline( is, inLine ) ){
@@ -117,14 +124,13 @@ namespace TiCC {
 	string::size_type pos = line.find("=");
 	if ( pos != string::npos ){
 	  string att = line.substr(0,pos);
+	  att = TiCC::trim(att);
 	  string val = line.substr(pos+1);
+	  val = TiCC::trim(val);
 	  if ( val[0] == '"' && val[val.length()-1] == '"' )
 	    val = val.substr(1, val.length()-2);
 	  val = fixControls( val );
 	  myMap[section][att] = val;
-	  if ( section == "global"
-	       && att == "configDir" )
-	    cdir = val;
 	}
 	else {
 	  cerr << "invalid attribute value pair in line '" << line << "'" << endl;
@@ -135,8 +141,8 @@ namespace TiCC {
     return true;
   }
 
-  bool Configuration::fill( const string& fileName, const string& section ){
-    ifstream is( fileName.c_str() );
+  bool Configuration::fill( const string& fileName, const string& insect ){
+    ifstream is( fileName );
     if ( !is ){
       cerr << "unable to read configuration from " << fileName << endl;
       return false;
@@ -144,7 +150,8 @@ namespace TiCC {
     bool found = false;
     string inLine;
     string localsection;
-    //  cerr << "looking for section = " << section << endl;
+    string section = TiCC::trim(insect);
+    //  cerr << "looking for section = " << insection << endl;
     while ( getline( is, inLine ) ){
       string line = TiCC::trim(inLine);
       if ( line.empty() )
@@ -155,6 +162,7 @@ namespace TiCC {
 	if ( line[line.length()-1] == ']' &&
 	     line[line.length()-2] == ']' ){
 	  localsection = line.substr(2,line.length()-4);
+	  localsection = TiCC::trim(localsection);
 	  //	cerr << "GOT section = " << localsection << endl;
 	}
 	else {
@@ -166,7 +174,9 @@ namespace TiCC {
 	string::size_type pos = line.find("=");
 	if ( pos != string::npos ){
 	  string att = line.substr(0,pos);
+	  att = TiCC::trim(att);
 	  string val = line.substr(pos+1);
+	  val = TiCC::trim(val);
 	  if ( val[0] == '"' && val[val.length()-1] == '"' )
 	    val = val.substr(1, val.length()-2);
 	  myMap[section][att] = val;
@@ -186,22 +196,22 @@ namespace TiCC {
   }
 
   void Configuration::dump( ostream& os ) const {
-    sssMap::const_iterator it1 = myMap.find("global");
+    auto it1 = myMap.find("global");
     if ( it1 == myMap.end() ){
       os << "empty" << endl;
       return;
     }
-    os << "[global]" << endl;
-    os << "configDir=" << cdir << endl;
-    ssMap::const_iterator it2 = it1->second.begin();
+    os << "[[global]]" << endl;
+    auto it2 = it1->second.begin();
     while ( it2 != it1->second.end() ){
+      string out = it2->second;
       os << it2->first << "=" << it2->second << endl;
       ++it2;
     }
     it1 = myMap.begin();
     while ( it1 != myMap.end() ){
       if ( it1->first != "global" ){
-	os << "[" << it1->first << "]" << endl;
+	os << endl << "[[" << it1->first << "]]" << endl;
 	it2 = it1->second.begin();
 	while ( it2 != it1->second.end() ){
 	  os << it2->first << "=" << it2->second << endl;
@@ -212,37 +222,80 @@ namespace TiCC {
     }
   }
 
-  string Configuration::setatt( const string& att,
-				const string& val,
-				const string& sect ){
+  void Configuration::create_configfile( const string& name ) const {
+    ofstream os( name );
+    if ( !os ){
+      throw runtime_error( "unable to create outputfile: " + name );
+    }
+    dump( os );
+  }
+
+  string Configuration::setatt( const string& inatt,
+				const string& inval,
+				const string& insect ){
     string oldVal;
-    string section = sect;
-    if ( section.empty() )
-      section = "global";
-    sssMap::iterator it1 = myMap.find( section );
+    string att = TiCC::trim(inatt);
+    string val = TiCC::trim(inval);
+    string sect = TiCC::trim(insect);
+    if ( sect.empty() )
+      sect = "global";
+    auto it1 = myMap.find( sect );
     if ( it1 != myMap.end() ){
-      ssMap::iterator it2 = it1->second.find( att );
+      auto const& it2 = it1->second.find( att );
       if ( it2 != it1->second.end() ){
 	oldVal = it2->second;
       }
       it1->second[att] = val;
     }
+    else {
+      myMap[sect].insert( make_pair( att, val ) );
+    }
     return oldVal;
   }
 
-  string Configuration::lookUp( const string& att, const string& section ) const {
-    sssMap::const_iterator it1;
-    if ( section.empty() )
-      it1 = myMap.find( "global" );
-    else
-      it1 = myMap.find( section );
+  string Configuration::clearatt( const string& inatt,
+				  const string& insect ){
+    //    cerr << "clear att: '" << inatt << "' in [" << insect << "]" << endl;
+    string oldVal;
+    string sect = TiCC::trim(insect);
+    string att = TiCC::trim(inatt);
+    if ( sect.empty() )
+      sect = "global";
+    //    cerr << "clear sect [" << sect << "]" << endl;
+    auto it1 = myMap.find( sect );
+    if ( it1 != myMap.end() ){
+      //      cerr << "found sect [" << sect << "]" << endl;
+      auto const& it2 = it1->second.find( att );
+      if ( it2 != it1->second.end() ){
+	//	cerr << "found att [" << att << "]" << endl;
+	oldVal = it2->second;
+      }
+      else {
+	//	cerr << "missed att [" << att << "]" << endl;
+      }
+      it1->second.erase( att );
+    }
+    else {
+      //      cerr << "MISSED sect [" << sect << "]" << endl;
+    }
+    return oldVal;
+  }
+
+  string Configuration::lookUp( const string& inatt,
+				const string& insect ) const {
+    string sect = TiCC::trim(insect);
+    string att = TiCC::trim(inatt);
+    string key = sect;
+    if ( key.empty() )
+      key = "global";
+    auto const& it1 = myMap.find( key );
     if ( it1 == myMap.end() ){
       return "";
     }
     else {
-      ssMap::const_iterator it2 = it1->second.find( att );
+      auto const& it2 = it1->second.find( att );
       if ( it2 == it1->second.end() ){
-	if ( section.empty() || section == "global" )
+	if ( sect.empty() || sect == "global" )
 	  return "";
 	else
 	  return lookUp( att, "global" );
@@ -252,15 +305,15 @@ namespace TiCC {
     }
   }
 
-  map<string,string> Configuration::lookUpAll( const string& section ) const {
+  map<string,string> Configuration::lookUpAll( const string& insect ) const {
     map<string,string> result;
-    sssMap::const_iterator it1;
-    if ( section.empty() )
-      it1 = myMap.find( "global" );
-    else
-      it1 = myMap.find( section );
+    string sect = TiCC::trim(insect);
+    if ( sect.empty() ){
+      sect = "global";
+    }
+    auto const& it1 = myMap.find( sect );
     if ( it1 != myMap.end() ){
-      ssMap::const_iterator it2 = it1->second.begin();
+      auto it2 = it1->second.begin();
       while ( it2 != it1->second.end() ){
 	result[it2->first] = it2->second;
 	++it2;
@@ -272,7 +325,7 @@ namespace TiCC {
   set<string> Configuration::lookUpSections() const {
     set<string> result;
     result.insert("global");
-    sssMap::const_iterator it = myMap.begin();
+    auto it = myMap.begin();
     while ( it != myMap.end() ){
       result.insert( it->first );
       ++it;
@@ -280,10 +333,11 @@ namespace TiCC {
     return result;
   }
 
-  bool Configuration::hasSection( const string& section ) const {
-    if ( !section.empty() ){
-      sssMap::const_iterator it1 = myMap.find( section );
-      if ( it1 != myMap.end() )
+  bool Configuration::hasSection( const string& insect ) const {
+    string sect = TiCC::trim( insect );
+    if ( !sect.empty() ){
+      auto const it = myMap.find( sect );
+      if ( it != myMap.end() )
 	return true;
     }
     return false;
